import std/[options, os, strutils, unicode]
import namespaced_logging, zero_functional

from fiber_orm import enableDbLogging

export namespaced_logging

var logService* {.threadvar.}: Option[LogService]

proc enableLogging*(svc: LogService = initLogService(), debug = false): LogService =
  if not (svc.cfg.appenders --> exists(it of ConsoleLogAppender)):
    svc.addAppender(initConsoleLogAppender(threshold = lvlAll))

  enableDbLogging(svc)
  logService = some(svc)
  result = svc


proc configureLoggingThresholds*(debug = false) =
  if not logService.isSome: return
  let logSvc = logService.get

  if debug:
    logSvc.cfg.rootLevel = Level.lvlDebug
    logSvc.cfg.loggers.add([
      LoggerConfig(name: "wdiwtlt", threshold: some(Level.lvlDebug)),
      LoggerConfig(name: "fiber_orm", threshold: some(Level.lvlDebug)),
    ])
  else: logSvc.cfg.rootLevel = Level.lvlInfo

  logSvc.reloadThreadState()


proc enableLoggingByEnvVar*(envVar = "DEBUG"): void =
  if not logService.isSome: discard enableLogging()
  let val = getEnv(envVar, "false").toLower()

  configureLoggingThresholds(
   "true".startsWith(val) or
   "yes".startsWith(val) or
   "on".startsWith(val) or
    val == "1")

proc getLogger*(name: string): Option[Logger] =
  if logService.isSome: return some(logService.get.getLogger(name))
  else: return none[Logger]()
