type ScrollText* = ref object
  text: string
  maxWidth: int
  scrollIdx: int        # Current index of the first character to show.
  lastRender: string

proc render(st: ScrollText): string =
  if st.text.len <= st.maxWidth:
    return st.text

  if st.scrollIdx == 0: return st.text[0..<st.maxWidth]
  elif st.scrollIdx == st.text.len:
    return " " & st.text[0..<st.maxWidth - 1]
  elif st.scrollIdx + st.maxWidth < st.text.len:
    return st.text[st.scrollIdx..<(st.scrollIdx + st.maxWidth)]
  else:
    return st.text[st.scrollIdx..<st.text.len] & " " &
      st.text[0..<max(0, (st.scrollIdx + st.maxWidth - st.text.len - 1))]

proc `text=`*(st: var ScrollText, text: string) =
  st.text = text
  st.scrollIdx = 0
  st.lastRender = render(st)

proc `maxWidth=`*(st: var ScrollText, maxWidth: int) =
  st.maxWidth = max(maxWidth, 1)
  st.lastRender = render(st)

proc initScrollText*(text: string, maxWidth: int): ScrollText =
  result = ScrollText(
    text: text,
    maxWidth: maxWidth,
    scrollIdx: 0)

  result.lastRender = render(result)

proc nextTick*(st: var ScrollText): string =

  st.lastRender = render(st)

  # Advance the scroll index by one.
  st.scrollIdx = (st.scrollIdx + 1) mod (st.text.len + 1)

  return st.lastRender

func `$`*(st: ScrollText): string = st.lastRender

converter toString*(st: ScrollText): string = st.lastRender
